@@ -4,7 +4,7 @@ package test;
 
 =head1 NAME
 
-test - tools for helping in test suites (not including running external programs).
+test - tools for helping in test suites (not including running externalprograms).
 
 =head1 SYNOPSIS
 
@@ -71,9 +71,9 @@ Unless TEST_DEBUG is set in the environment.
 
 # Pragmas -----------------------------
 
-use v5.6.1;
+use 5.00503;
 use strict;
-use warnings;
+use vars qw( @EXPORT_OK );
 
 # Inheritance -------------------------
 
@@ -109,29 +109,54 @@ The following symbols are exported upon request:
 
 =item tempdir
 
+=item find_exec
+
+=item read_file
+
 =back
 
 =cut
 
-our @EXPORT_OK = qw( BIN_DIR DATA_DIR REF_DIR LIB_DIR
-                     check_req compare evcheck only_files
-                     save_output restore_output tempdir tmpnam );
+@EXPORT_OK = qw( BIN_DIR DATA_DIR REF_DIR LIB_DIR
+                 check_req compare evcheck find_exec only_files read_file
+                 save_output restore_output tempdir tmpnam );
 
 # Utility -----------------------------
 
 use Carp                          qw( carp croak );
-use Env                           qw( @PERL5LIB );
-use Fatal                    1.02 qw( :void close open seek sysopen unlink );
-use Fcntl                    1.03 qw( :DEFAULT :seek );
+use Cwd                      2.01 qw( cwd );
+use Env                           qw( PATH PERL5LIB );
+use Fatal                    1.02 qw( close open seek sysopen unlink );
+use Fcntl                    1.03 qw( :DEFAULT );
 use File::Compare          1.1002 qw( );
-use File::Path             1.0403 qw( mkpath rmtree );
-use File::Spec::Functions         qw( catdir catfile rel2abs updir );
-use File::Temp               0.12 qw( tempfile );
+use File::Path             1.0401 qw( mkpath rmtree );
+use File::Spec                0.6 qw( );
 use FindBin                  1.42 qw( $Bin );
-use Test                     1.13 qw( ok skip );
+use POSIX                    1.02 qw( );
+use Test                    1.122 qw( ok skip );
 
 # ----------------------------------------------------------------------------
 
+sub rel2abs {
+  if ( File::Spec->file_name_is_absolute($_[0]) ) {
+    return $_[0];
+  } else {
+    return catdir(cwd, $_[0]);
+  }
+}
+
+sub catdir {
+  File::Spec->catdir(@_);
+}
+
+sub catfile {
+  File::Spec->catfile(@_);
+}
+
+sub updir {
+  File::Spec->updir(@_);
+}
+
 # -------------------------------------
 # PACKAGE CONSTANTS
 # -------------------------------------
@@ -145,10 +170,12 @@ use constant LIB_DIR  => catdir $Bin, updir, 'lib';
 # PACKAGE ACTIONS
 # -------------------------------------
 
-unshift @PERL5LIB, LIB_DIR;
+# @PERL5LIB not available in Env for perl 5.00503
+# unshift @PERL5LIB, LIB_DIR;
+$PERL5LIB = defined $PERL5LIB ? join(':', LIB_DIR, $PERL5LIB) : LIB_DIR;
 unshift @INC,      LIB_DIR;
 
-$_ = rel2abs $_
+$_ = rel2abs($_)
   for @INC;
 
 my $tmpdn = tempdir();
@@ -157,10 +184,9 @@ $| = 1;
 mkpath $tmpdn;
 die "Couldn't create temp dir: $tmpdn: $!\n"
   unless -r $tmpdn and -w $tmpdn and -x $tmpdn and -o $tmpdn and -d $tmpdn;
-CHECK { # Don't do this until all modules compiled
-  # Perhaps we should actually resolve all @INC to absolute paths beforehand?
-  chdir $tmpdn;
-}
+
+#@INC = map rel2abs($_), @INC;
+chdir $tmpdn;
 
 # -------------------------------------
 # PACKAGE FUNCTIONS
@@ -197,9 +223,10 @@ Arrayref of names of files to expect to exist.
 sub only_files {
   my ($expect) = @_;
 
-  opendir my $dir, '.';
-  my %files = map { $_ => 1 } readdir $dir;
-  closedir $dir;
+  local *MYDIR;
+  opendir MYDIR, '.';
+  my %files = map { $_ => 1 } readdir MYDIR;
+  closedir MYDIR;
 
   my $ok = 1;
 
@@ -317,10 +344,16 @@ sub save_output {
     unless @_ == 2;
   my ($name, $filehandle) = @_;
 
-  my $tmpfh = tempfile;
+  my $tmpfh  = do { local *F; *F; };
+  my $savefh = do { local *F; *F; };
+
+  my $tmpnam = POSIX::tmpnam;
+  sysopen $tmpfh, $tmpnam, O_RDWR | O_CREAT | O_EXCL;
+  unlink $tmpnam;
   select((select($tmpfh), $| = 1)[0]);
 
-  open my $savefh, '>&' . fileno $filehandle
+
+  open $savefh, '>&' . fileno $filehandle
     or die "can't dup $name: $!";
   open $filehandle, '>&' . fileno $tmpfh
     or die "can't open $name to tempfile: $!";
@@ -372,7 +405,7 @@ sub restore_output {
   open  $origfh, '>&' . fileno $savefh
     or die "cannot dup $name back again: $!";
 
-  seek $tmpfh, 0, SEEK_SET;
+  seek $tmpfh, 0, 0;
   local $/ = undef;
   my $string = <$tmpfh>;
   close $tmpfh;
@@ -428,20 +461,33 @@ context.
 =cut
 
 my @tmpfns;
-sub tmpnam {
-  my $tmpnam = File::Temp::tmpnam;
 
-  if (@_) {
-    push @tmpfns, [ $tmpnam, $_[0] ];
-  } else {
-    push @tmpfns, $tmpnam;
-  }
+BEGIN {
+  my $savewarn = $SIG{__WARN__};
+  # Subvert bizarre (& incorrect) subroutine redefined errors in 5.005_03
+  local $SIG{__WARN__} =
+    sub {
+      $savewarn->(@_)
+        if defined $savewarn                        and
+           UNIVERSAL::isa($savewarn,'CODE')         and
+           $_[0] !~ /^Subroutine tmpnam redefined/;
+    };
 
-  if (wantarray) {
-    sysopen my $tmpfh, $tmpnam, O_RDWR | O_CREAT | O_EXCL;
-    return $tmpnam, $tmpfh;
-  } else {
-    return $tmpnam;
+  *tmpnam = sub {
+    my $tmpnam = POSIX::tmpnam;
+
+    if (@_) {
+      push @tmpfns, [ $tmpnam, $_[0] ];
+    } else {
+      push @tmpfns, $tmpnam;
+    }
+
+    if (wantarray) {
+      sysopen my $tmpfh, $tmpnam, O_RDWR | O_CREAT | O_EXCL;
+      return $tmpnam, $tmpfh;
+    } else {
+      return $tmpnam;
+    }
   }
 }
 
@@ -455,7 +501,8 @@ END {
       }
     }
   } else {
-    unlink map((ref $_ ? $_->[0] : $_), @tmpfns);
+    unlink map((ref $_ ? $_->[0] : $_), @tmpfns)
+      if @tmpfns;
   }
 }
 
@@ -488,8 +535,9 @@ Name of temporary dir.
 
 my @tmpdirs;
 sub tempdir {
-  my $cleanup = ! (defined $ENV{TEST_DEBUG} && $ENV{TEST_DEBUG} =~ /\bSAVE\b/);
-  my $tempdir = File::Temp::tempdir(CLEANUP => $cleanup);
+  my $tempdir = POSIX::tmpnam;
+  mkdir $tempdir, 0700
+    or die "Failed to create temporary directory $tempdir: $!\n";
 
   if (@_) {
     push @tmpdirs, [ $tempdir, $_[0] ];
@@ -501,12 +549,18 @@ sub tempdir {
 }
 
 END {
-  if ( defined $ENV{TEST_DEBUG} and $ENV{TEST_DEBUG} =~ /\bSAVE\b/ ) {
-    for (@tmpdirs) {
-      if ( ref $_ ) {
+  for (@tmpdirs) {
+    if ( ref $_ ) {
+      if ( defined $ENV{TEST_DEBUG} and $ENV{TEST_DEBUG} =~ /\bSAVE\b/ ) {
         printf "Used temp dir: %s (%s)\n", @$_;
       } else {
+        rmtree $_->[0];
+      }
+    } else {
+      if ( defined $ENV{TEST_DEBUG} and $ENV{TEST_DEBUG} =~ /\bSAVE\b/ ) {
         print "Used temp dir: $_\n";
+      } else {
+        rmtree $_;
       }
     }
   }
@@ -516,94 +570,316 @@ END {
 
 =head2 compare
 
-  ok compare($fn1, $fn2), 1, 'test name';
+  compare(+{ fn1 => $fn1, fn2 => $fn2, gzip => 1 }), 1, 'test name';
 
-Compare two files, return 1 if they are the same, 0 if they differ, -1 if file
-cannot be read, -2 if file 2 cannot be read, -4 if both cannot be read.  -8
-for a usage error.
+This performs one test.
 
-In TEST_DEBUG mode, if the files do not compare, outputs file info on STDERR.
+=over 4
+
+=item ARGUMENTS
+
+A single argument is taken, considered as a hash ref, with the following keys:
+
+In TEST_DEBUG mode, if the files do not compare equal, outputs file info on
+STDERR.
+
+=over 4
+
+=item fn1
+
+B<Mandatory>  File to compare
+
+=item fn2
+
+B<Mandatory>  File to compare
+
+=item name
+
+B<Mandatory>  Test name
+
+=item sort
+
+B<Optional> sort files prior to comparison.  Requires the C<sort> command to
+be on C<$PATH> (else skips).
+
+=item gunzip
+
+B<Optional> gunzip files prior to comparison.  Requires the C<gzip> command to
+be on C<$PATH> (else skips).  gzip occurs prior to any sort.
+
+=item untar
+
+B<Optional> untar files prior to comparison.  Requires the C<tar> command to
+be on C<$PATH> (else skips).  any gzip occurs prior to any tar.  Tar files are
+considered equal if they each contain the same filenames & each file contained
+is equal.  If the sort flag is present, each file is sorted prior to comparison.
+
+=back
+
+=back
 
 =cut
 
-sub compare {
-  my ($fn1, $fn2) = @_;
+#XYZ sub _run {
+#XYZ   my ($cmd, $name, $in) = @_;
+#XYZ
+#XYZ   my $infn = defined $in ? tmpnam : '/dev/null';
+#XYZ   my $outfn = tmpnam;
+#XYZ   my $errfn = tmpnam;
+#XYZ
+#XYZ   my $pid = fork;
+#XYZ   croak "Couldn't fork: $!\n"
+#XYZ     unless defined $pid;
+#XYZ
+#XYZ   if ( $pid == 0 ) { # Child
+#XYZ     open STDOUT, '>', $outfn;
+#XYZ     open STDERR, '>', $errfn;
+#XYZ     open STDIN,  '<', $infn;
+#XYZ
+#XYZ     exec @$cmd;
+#XYZ   }
+#XYZ
+#XYZ   my $rv = waitpid $pid, 0;
+#XYZ   my $status = $?;
+#XYZ
+#XYZ   croak "Unexpected waitpid return from child $name: $rv (expected $pid)\n"
+#XYZ     unless $rv == $pid;
+#XYZ
+#XYZ   local $/ = undef;
+#XYZ   local (OUT, ERR);
+#XYZ   open *OUT, '<', $outfn;
+#XYZ   open *ERR, '<', $errfn;
+#XYZ   my $out = <OUT>;
+#XYZ   my $err = <ERR>;
+#XYZ   close *OUT;
+#XYZ   close *ERR;
+#XYZ
+#XYZ   return $status >> 8, $status & 127, $status & 128 , $out, $err
+#XYZ }
 
-  for ( grep ! defined, $fn1, $fn2 ) {
-    carp "Usage: compare($fn1, $fn2)\n"
-        if $ENV{TEST_DEBUG};
-    return -8;
-  }
+# return codes and old-style call semantics left for backwards compatibility
+BEGIN {
+  my $savewarn = $SIG{__WARN__};
+  # Subvert bizarre (& incorrect) subroutine redefined errors in 5.005_03
+  local $SIG{__WARN__} =
+    sub {
+      $savewarn->(@_)
+        if defined $savewarn                        and
+           UNIVERSAL::isa($savewarn,'CODE')         and
+           $_[0] !~ /^Subroutine compare redefined/;
+    };
 
-  {
-    my $err = 0;
+  *compare = sub {
+    my ($fn1, $fn2, $sort) = @_;
+    my ($gzip, $tar, $name);
+    my $notest = 1;
 
-    for (0..1) {
-      my $fn = ($fn1, $fn2)[$_];
-      if ( ! -e $fn ) {
-        carp "Does not exist: $fn\n"
+    if ( @_ == 1 and UNIVERSAL::isa($_[0], 'HASH') ) {
+      ($fn1, $fn2, $name, $sort, $gzip, $tar, $notest) =
+        @{$_[0]}{qw( fn1 fn2 name sort gunzip untar notest )};
+      my @missing = grep ! defined $_[0]->{$_}, qw( fn1 fn2 name );
+      carp "Missing mandatory key(s): " . join(', ', @missing) . "\n"
+        if @missing;
+    }
+
+    my ($name1, $name2) = ($fn1, $fn2);
+
+    for ( grep ! defined, $fn1, $fn2 ) {
+      carp 'Usage: compare({fn1 => $fn1, fn2 => $fn2, name => "some name"})' ."\n"
           if $ENV{TEST_DEBUG};
-        $err &= 2 ** $_;
-      } elsif ( ! -r $fn ) {
-        carp "Cannot read: $fn\n"
+      ok 0, 1, $name
+        unless $notest;
+      return -8;
+    }
+
+    {
+      my $err = 0;
+
+      for (0..1) {
+        my $fn = ($name1, $name2)[$_];
+        if ( ! -e $fn ) {
+          carp "Does not exist: $fn\n"
+            if $ENV{TEST_DEBUG};
+          $err |= 2 ** $_;
+        } elsif ( ! -r $fn ) {
+          carp "Cannot read: $fn\n"
+            if $ENV{TEST_DEBUG};
+          $err |= 2 ** $_;
+        }
+      }
+
+      if ( $err ) {
+        ok 0, 1, $name
+          unless $notest;
+        return -$err;
+      }
+    }
+
+    if ( $gzip ) {
+      unless ( find_exec('gzip') ) {
+        print "ok # Skip gzip not found in path\n";
+        return -16;
+      }
+
+      my $tmp1 = tmpnam;
+      my $tmp2 = tmpnam;
+      system "gzip $fn1 -cd > $tmp1"
+        and croak "gzip $fn1 failed: $?\n";
+      system "gzip $fn2 -cd > $tmp2"
+        and croak "gzip $fn2 failed: $?\n";
+      ($fn1, $fn2) = ($tmp1, $tmp2);
+    }
+
+    if ( $tar ) {
+      unless ( find_exec('tar') ) {
+        print "ok # Skip tar not found in path\n";
+        return -16;
+      }
+
+      local $/ = "\n";
+      chomp (my @list1 = sort qx( tar tf $fn1 ));
+      croak "tar tf $fn1 failed with wait status: $?\n"
+        if $?;
+      chomp(my @list2 = sort qx( tar tf $fn2 ));
+      croak "tar tf $fn2 failed with wait status: $?\n"
+        if $?;
+
+      if ( @list2 > @list1 ) {
+        carp
+          sprintf("More files (%d) in $name2 than $name1 (%d)\n",
+                  scalar @list2, scalar @list1)
           if $ENV{TEST_DEBUG};
-        $err &= 2 ** $_;
+        ok @list1, @list2, $name
+          unless $notest;
+        return 0;
+      } elsif ( @list1 > @list2 ) {
+        carp
+          sprintf("More files (%d) in $name1 than $name2 (%d)\n",
+                  scalar @list1, scalar @list2)
+          if $ENV{TEST_DEBUG};
+        ok @list1, @list2, $name
+          unless $notest;
+        return 0;
       }
+
+      for (my $i = 0; $i < @list1; $i++) {
+        if ( $list1[$i] lt $list2[$i] ) {
+          carp "File $list1[$i] is present in $name1 but not $name2\n"
+            if $ENV{TEST_DEBUG};
+          ok $list1[$i], $list2[$i], $name
+            unless $notest;
+          return 0;
+        } elsif ( $list1[$i] gt $list2[$i] ) {
+          carp "File $list2[$i] is present in $name2 but not $name1\n"
+            if $ENV{TEST_DEBUG};
+          ok $list2[$i], $list1[$i], $name
+            unless $notest;
+          return 0;
+        }
+      }
+
+      for my $fn (@list1) {
+        my $tmp1 = tmpnam;
+        my $tmp2 = tmpnam;
+        system "tar -xf $fn1 -O $fn > $tmp1"
+          and croak "tar -xf $fn1 -O $fn failed: $?\n";
+        system "tar -xf $fn2 -O $fn > $tmp2"
+          and croak "tar -xf $fn2 -O $fn failed: $?\n";
+        my $ok = compare({ fn1    => $tmp1,
+                           fn2    => $tmp2,
+                           sort   => $sort,
+                           notest => 1,
+                           name   =>
+                             qq'Subcheck file "$fn" for compare $name1, $name2',
+                         });
+        unless ( $ok >= 1 ) {
+          carp qq'Difference found testing file "$fn" in tars $name1 ($tmp1), $name2 ($tmp2)\n'
+            if $ENV{TEST_DEBUG};
+          ok 0, 1, $name
+            unless $notest;
+          return 0;
+        }
+      }
+
+      ok 1, 1, $name
+        unless $notest;
+      return 1;
     }
 
-    return -$err
-      if $err;
+    if ( $sort ) {
+      unless ( find_exec('sort') ) {
+        print "ok # Skip sort not found in path\n";
+        return -16;
+      }
+
+      my $tmp1 = tmpnam;
+      my $tmp2 = tmpnam;
+      system sort => $fn1, -o => $tmp1
+        and croak "Sort $fn1 failed: $?\n";
+      system sort => $fn2, -o => $tmp2
+        and croak "Sort $fn2 failed: $?\n";
+      ($fn1, $fn2) = ($tmp1, $tmp2);
+    }
+
+    unless ( File::Compare::compare($fn1, $fn2) ) {
+      ok 1, 1, $name
+        unless $notest;
+      return 1;
+    }
+
+    if ( $ENV{TEST_DEBUG} ) {
+      my $pid = fork;
+      die "Fork failed: $!\n"
+        unless defined $pid;
+
+      if ( $pid ) { # Parent
+        my $waitpid = waitpid($pid, 0);
+        die "Waitpid got: $waitpid (expected $pid)\n"
+          unless $waitpid == $pid;
+      } else { # Child
+        open *STDOUT{IO}, ">&" . fileno STDERR;
+        # Uniquify file names
+        my @args = keys %{+{ map {;$_=>1} $name1, $name2, $fn1, $fn2 }};
+        exec qw(ls -l), @args;
+      }
+
+      my $fh1 = IO::File->new($fn1, O_RDONLY)
+        or die "Couldn't open $fn1: $!\n";
+      my $fh2 = IO::File->new($fn2, O_RDONLY)
+        or die "Couldn't open $fn2: $!\n";
+
+      local $/ = "\n";
+
+      my $found = 0;
+      while ( ! $found and my $line1 = <$fh1> ) {
+        my $line2 = <$fh2>;
+        if ( ! defined $line2 ) {
+          print STDERR "$fn2 ended at line: $.\n";
+          $found = 1;
+        } elsif ( $line2 ne $line1 ) {
+          print STDERR
+            "Difference at line $.:\n-$name1->$line1<---\n-$name2->$line2<---\n-";
+          $found = 1;
+        }
+      }
+
+      if ( ! $found ) {
+        my $line2 = <$fh2>;
+        if ( defined $line2 ) {
+          print STDERR "$name1 ended before line: $.\n";
+        } else {
+          print STDERR "Difference between $name1, $name2 not found!\n";
+        }
+      }
+
+      close $fh1;
+      close $fh2;
+    }
+
+    ok 0, 1, $name
+      unless $notest;
+    return 0;
   }
-
-  return 1
-    unless File::Compare::compare($fn1, $fn2);
-
-  if ( $ENV{TEST_DEBUG} ) {
-    my $pid = fork;
-    die "Fork failed: $!\n"
-      unless defined $pid;
-
-    if ( $pid ) { # Parent
-      my $waitpid = waitpid($pid, 0);
-      die "Waitpid got: $waitpid (expected $pid)\n"
-        unless $waitpid == $pid;
-    } else { # Child
-      open *STDOUT{IO}, ">&" . fileno STDERR;
-      exec qw(ls -l), $fn1, $fn2;
-    }
-
-    open my $fh1, '<', $fn1 or die "$fn1: $!\n";
-    open my $fh2, '<', $fn2 or die "$fn2: $!\n";
-
-    local $/ = "\n";
-
-    my $found = 0;
-    while ( ! $found and my $line1 = <$fh1> ) {
-      my $line2 = <$fh2>;
-      if ( ! defined $line2 ) {
-        print STDERR "$fn2 ended at line: $.\n";
-        $found = 1;
-      } elsif ( $line2 ne $line1 ) {
-        print STDERR
-          "Difference at line $.:\n-1->$line1<---\n-2->$line2<---\n-";
-        $found = 1;
-      }
-    }
-
-    if ( ! $found ) {
-      my $line2 = <$fh2>;
-      if ( defined $line2 ) {
-        print STDERR "$fn1 ended before line: $.\n";
-      } else {
-        print STDERR "Difference between $fn1, $fn2 not found!\n";
-      }
-    }
-
-    close $fh1;
-    close $fh2;
-  }
-
-  return 0;
 }
 
 # -------------------------------------
@@ -720,6 +996,98 @@ sub check_req {
   $test->(only_files([]), 1);
 }
 
+# -------------------------------------
+
+=head2 find_exec
+
+=over 4
+
+=item ARGUMENTS
+
+=over 4
+
+=item proggie
+
+The name of the program
+
+=back
+
+=item RETURNS
+
+=over 4
+
+=item path
+
+The path to the first executable file with the given name on C<$PATH>.  Or
+nothing, if no such file exists.
+
+=back
+
+=back
+
+=cut
+
+sub find_exec {
+  my ($exec) = @_;
+
+  for (split /:/, $PATH) {
+    my $try = catfile $_, $exec;
+    return $try
+      if -x $try;
+  }
+  return;
+}
+
+# -------------------------------------
+
+=head2 read_file
+
+=over 4
+
+=item ARGUMENTS
+
+=over 4
+
+=item filename
+
+B<Mandatory>
+
+=item line-terminator
+
+B<Optional>.  Value of C<$/>.  Defaults to C<"\n">.
+
+=back
+
+=item RETURNS
+
+=over 4
+
+=item lines
+
+A list of lines in the file (lines determined by the value of
+line-terminator), as an arrayref.
+
+=back
+
+=back
+
+=cut
+
+sub read_file {
+  my ($fn, $term) = @_;
+
+  $term = "\n"
+    unless defined $term;
+
+  my $fh = do { local *F; *F };
+  sysopen $fh, $fn, O_RDONLY;
+  local $/ = $term;
+  my @lines = <$fh>;
+  close $fh;
+
+  return \@lines;
+}
+
 # ----------------------------------------------------------------------------
 
 =head1 EXAMPLES
